﻿#ADのグループにユーザーが含まれているか確認する
function Test-UserIsInADGroup {
    param (
        [Parameter(Mandatory)][string]$GroupName,
        [Parameter(Mandatory)][string]$Domain,
        [Parameter(Mandatory)][string]$UserName
    )

    try {
        Add-Type -AssemblyName System.DirectoryServices
        Add-Type -AssemblyName System.DirectoryServices.AccountManagement

        # ユーザーが指定したグループに含まれているかをチェック
        $domainContext = New-Object -TypeName System.DirectoryServices.AccountManagement.PrincipalContext -ArgumentList 'Domain', $Domain
        $localContext = New-Object -TypeName System.DirectoryServices.AccountManagement.PrincipalContext -ArgumentList 'Machine'

        $userPrincipal = [System.DirectoryServices.AccountManagement.UserPrincipal]::FindByIdentity($domainContext, $UserName)
        $localUserPrincipal = [System.DirectoryServices.AccountManagement.UserPrincipal]::FindByIdentity($localContext, $UserName)

        if ($null -ne $userPrincipal) {
            $groupPrincipal = [System.DirectoryServices.AccountManagement.GroupPrincipal]::FindByIdentity($domainContext, $GroupName)
            $localGroupPrincipal = [System.DirectoryServices.AccountManagement.GroupPrincipal]::FindByIdentity($localContext, $GroupName)

            if (($null -ne $groupPrincipal -and $userPrincipal.IsMemberOf($groupPrincipal)) -or
                ($null -ne $localGroupPrincipal -and $localUserPrincipal.IsMemberOf($localGroupPrincipal))) {
                return $true
            }
        }
    } catch {
        # エラーの場合は$falseを返す
    }
    return $false
}