﻿function Resolve-IPAddress {
    param(
        [Parameter(Mandatory)][string]$Target
    )

    # 入力がIPアドレスの形式かどうかをチェック
    if (Test-IPaddress -IPAddress $Target) {
        return $Target
    }

    # 入力がホスト名の場合、IPアドレスを解決して返す
    try {
        $ipAddress = [System.Net.Dns]::GetHostAddresses($Target) |
            Where-Object { ($_.AddressFamily -eq 'InterNetwork') -or ($_.AddressFamily -eq 'InterNetworkV6') }
        if ($ipAddress) {
            return $ipAddress.IPAddressToString
        }
    } catch {
        #        Write-Host "ホスト名 '$Target' を解決できませんでした。"
    }

    return $null
}


function Test-TcpPortOpen {
    param (
        [Parameter(Mandatory)][string]$Target,
        [Parameter(Mandatory)][int]$Port
    )

    # IPアドレスが取得できない場合は接続できないとする
    $ips = Resolve-IPAddress -Target $Target
    if ($null -eq $ips) {
        return $false
    }

    foreach ($ip in $ips) {
        $ipAddress = [System.Net.IPAddress]::Parse($ip)
        $tcpClient = [System.Net.Sockets.TcpClient]::new($ipAddress.AddressFamily)
        try {
            $tcpClient.Connect($Target, $port)
        } catch {
            #        Write-Host "${Target}:${Port} 接続に失敗しました。:$($_.Exception.Message)"
            continue
        }
        $tcpClient.Close()
        return $true
    }
    return $false
}

function Test-DnsResolution {
    param (
        [Parameter(Mandatory)][string]$hostname
    )

    try {
        $null = Resolve-DnsName -Name $hostname -ErrorAction Stop
        return $true
    } catch {
        #        Write-Host "$hostname の名前解決に失敗しました。:$($_.Exception.Message)"
        return $false
    }
}

function Test-IPaddress {
    param (
        [Parameter(Mandatory)][string]$IPAddress
    )

    try {
        [ipaddress]$IPAddress
        return $true
    } catch {
        return $false
    }
}