﻿class UserRightsAssignment {
    # 取得する権限名
    # https://learn.microsoft.com/en-us/windows/security/threat-protection/security-policy-settings/user-rights-assignment
    $privileges = @(
        "SeAssignPrimaryTokenPrivilege", #プロセスレベルトークンの置き換え
        "SeIncreaseQuotaPrivilege"#プロセスのメモリクォータの増加
    )

    # 権限とSIDを保持する連想配列
    $privileges_sides = @{}

    # コンストラクタ
    UserRightsAssignment() {
        # 取得結果を格納する変数の初期化
        foreach ($key in $this.privileges) {
            $this.privileges_sides[$key] = @()
        }

        # secedit.exeコマンドの実行結果を一時ファイル書き出すためのパス
        $tempFile = [System.IO.Path]::GetTempFileName()

        $this.LoadPrivileges($tempFile)

        if (Test-Path $tempFile) {
            Remove-Item $tempFile -Force
        }
    }

    # secedit.exeコマンドでユーザー権限一覧を取得し
    # 必要な権限を持っているSIDを保持する
    LoadPrivileges ([string]$tempFile) {
        # secedit.exeコマンドの実行
        Start-Process -FilePath secedit.exe -ArgumentList "/export /areas USER_RIGHTS /cfg $tempFile" -Wait -WindowStyle Hidden

        foreach ($privilege in $this.privileges) {
            # ファイルを取得する
            Get-Content $tempFile -Encoding Unicode | Where-Object {
                # Where the privilege is listed.
                $_ -match $privilege
            } | ForEach-Object {
                # Separate the key and values.
                $_.split('=')[1].split(',') | ForEach-Object {

                    # Output the user/SID values
                    $_.trim()

                    $this.privileges_sides[$privilege] += $_
                }
            }
        }
    }

    # 指定されたSIDが該当の権限を有しているかを返す
    [bool] HasPrivilege([string]$privilegeKey, [string]$sid) {
        if ($sid.Length -le 0) {
            return $false
        }

        foreach ($prv in $this.privileges_sides[$privilegeKey]) {
            $hit_idx = $prv.IndexOf($sid)
            if ($hit_idx -ge 0) {
                return $true
            }
        }
        return $false
    }

    # 「プロセスレベルトークンの置き換え」がユーザーに付与されているかを返す
    [bool] HasProcessTokenPrivilege([string]$sid) {
        return $this.HasPrivilege("SeAssignPrimaryTokenPrivilege", $sid)
    }

    # 「プロセスのメモリクォータの増加」がユーザーに付与されているかを返す
    [bool] HasIncreaseQuotaPrivilege([string]$sid) {
        return $this.HasPrivilege("SeIncreaseQuotaPrivilege", $sid)
    }
}



