﻿$here = Split-Path -Parent $MyInvocation.MyCommand.Path
. "$here\Registry.ps1"
. "$here\Common.ps1"


#管理共有レジストリが有効化されているか
# true:有効化されている/false:有効化されていない
function Test-AdminShareEnabled-Registry {
    param (
        [bool]$isBuildInAdministrator = $false, # ビルドインAdministratorユーザー指定
        [bool]$isClient = (IsClientOS) # クライアントOSかどうかを設定
    )

    $lanmanServerRegPath = "HKLM:\SYSTEM\CurrentControlSet\Services\LanmanServer\Parameters"

    # クライアントとサーバーで見るエントリが違う
    if ($isClient -eq $true) {
        $lanmanServerName = "AutoShareWks"
    } else {
        $lanmanServerName = "AutoShareServer"
    }

    $autoShareServer = Get-RegistryValue $lanmanServerRegPath $lanmanServerName

    # エントリが1またはnullの場合は管理共有が有効
    if ($autoShareServer -eq 1 -or $null -eq $autoShareServer) {
        #ビルドインAdministratorユーザーの場合は
        #リモートUACが無効でも無条件で接続できるので有効状態を返す
        if ($isBuildInAdministrator -eq $true) {
            return $true
        }

        # LocalAccountTokenFilterPolicyが0の場合はリモートUAC有効
        $localAccountTokenFilterPolicyRegPath = "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System"
        $localAccountTokenFilterPolicyName = "LocalAccountTokenFilterPolicy"

        $localAccountTokenFilterPolicy = Get-RegistryValue $localAccountTokenFilterPolicyRegPath $localAccountTokenFilterPolicyName
        if ($localAccountTokenFilterPolicy -eq 1) {
            # ビルドインAdministrator以外は管理共有が有効と判断
            return $true
        } else {
            return $false
        }
    } else {
        return $false
    }
}

# コマンドで管理共有が現在有効かどうかチェックする
function Test-AdminShareEnabled-Command {
    $Shares = @(Get-SmbShare | Where-Object { $_.Name -eq "ADMIN$" })

    # ADMIN$が存在すれば現在は管理共有は有効と判断
    if ($Shares.Count -gt 0) {
        return $true
    }

    return $false
}

#管理共有が有効化されているか
# true:有効化されている/false:有効化されていない
function Test-AdminShareEnabled {
    param (
        [bool]$isBuildInAdministrator = $false, # ビルドインAdministratorユーザー指定
        [bool]$isClient = (IsClientOS) # クライアントOSかどうかを設定
    )

    # 現在の管理共有状態をチェックして、有効でないならfalseを即座に返す。
    $enableNow = Test-AdminShareEnabled-Command
    if ($enableNow -eq $false) {
        return $false
    }

    # 最後にレジストリの状態も見て判定する
    return Test-AdminShareEnabled-Registry -isBuildInAdministrator $isBuildInAdministrator -isClient $isClient
}

# Guest認証設定確認
# ローカルセキュリティポリシー
# ローカルポリシー -> セキュリティオプション -> ネットワークアクセス:ローカルアカウントの共有とセキュリティモデル
# https://learn.microsoft.com/en-us/windows/security/threat-protection/security-policy-settings/network-access-sharing-and-security-model-for-local-accounts
# 設定がClassicであるかどうかを判定して返す
function Test-LocalAccountSharingClassic {
    $regPath = "HKLM:\SYSTEM\CurrentControlSet\Control\Lsa"
    $entryName = "ForceGuest"

    $forceGuest = Get-RegistryValue $regPath $entryName

    if ($null -eq $forceGuest -or $forceGuest -eq 0) {
        return $true
    }

    return $false
}
