﻿$here = Split-Path -Parent $MyInvocation.MyCommand.Path
. "$here\ActiveDirectory.ps1"


# ユーザー情報の取得
function Get-UserAccountInfo {
    param (
        [Parameter(Mandatory)][string]$Account
    )

    if ($Account -match '^(.*?)\\(.*)$') {
        $domainName, $userName = $Account -split '\\', 2
    } else {
        $domainName = $env:COMPUTERNAME
        $userName = $Account
    }

    $userAccount = Get-CimInstance Win32_UserAccount -Filter "Name = '$userName' AND Domain = '$domainName'"

    if ($userAccount) {
        return $userAccount
    } else {
        return $null
    }
}

# ユーザーのSIDを取得する
function Get-UserSid {
    param (
        [Parameter(Mandatory)][string]$userName
    )

    $userSid = (New-Object System.Security.Principal.NTAccount($userName)).Translate([System.Security.Principal.SecurityIdentifier]).Value
    return $userSid
}


# 指定したグループにユーザーが属しているかチェックする
function Test-UserIsInGroup {
    param (
        [Parameter(Mandatory)][string]$GroupName, # グループ名
        [Parameter(Mandatory)]$UserAccount# ユーザー情報
    )
    # 指定されたグループに登録されているグループとユーザーを取得する
    $group_infos = Get-LocalGroupMember -Name $GroupName -ErrorAction SilentlyContinue

    #取得したグループの中にユーザーのSIDが存在するか確認する。
    $userIsAdministrator = $null -ne ($group_infos | Where-Object { $_.SID.Value -eq $UserAccount.SID })

    # ユーザーのSIDが存在したら属していると判定
    if ($userIsAdministrator) {
        return $true
    }

    # ADのグループかどうかチェックする
    foreach ($g in $group_infos) {
        # AD向けのグループチェックする
        # ObjectClassは日本語環境だと日本語の”グループ”で返ってきてしまうので英語とOrでチェック
        if (($g.PrincipalSource -eq "ActiveDirectory") -And
        (($g.ObjectClass -eq "グループ") -or ($g.ObjectClass -eq "Group"))) {
            if ($true -eq (Test-UserIsInADGroup -GroupName $g.Name -Domain $UserAccount.Domain -UserName $UserAccount.Name)) {
                return $true
            }
        }
    }

    return $false
}

# 指定ユーザーがAdministratorsグループに属しているチェックする。
# true:属している/false:属していない
function Test-AdministratorGroupMembership {
    param (
        [Parameter(Mandatory)]$UserAccount# ユーザー情報
    )

    try {
        # Administratorsグループが存在するか確認
        $null = (New-Object System.Security.Principal.NTAccount("Administrators")).Translate([System.Security.Principal.SecurityIdentifier]).Value

        # ユーザーがAdministratorsグループのメンバーかどうかを確認
        return Test-UserIsInGroup -GroupName "Administrators" -UserAccount $UserAccount
    } catch {
        #        Write-Host "指定ユーザーはAdministratorsグループに属していませんでした。:$($_.Exception.Message)"
        return $false
    }
}

# ビルドインadministratorかどうかをチェックする
# true:ビルドインadministrator/false:違う
function Test-BuildInAdministrator {
    param (
        [ValidateLength(1, 256)]
        [Parameter(Mandatory)][string]$sid
    )

    if ($sid.StartsWith("S-1-5-") -and $sid.EndsWith("-500")) {
        return $true
    }
    return $false
}