﻿Param(
    # domain 255, \ 1, user 255
    [parameter(mandatory = $true)][ValidateLength(1, 512)][string]$account,
    [parameter(mandatory = $true)][string]$remote
)


#共通処理の読み込み
. ..\common\common.ps1
. ..\common\User.ps1
. ..\common\Priviledge.ps1
. ..\common\NetWork.ps1
. ..\common\Uac.ps1

# セキュリティポリシーのチェック
function Test-SecurityPolicy {
    param(
        [Parameter(Mandatory)][UserRightsAssignment]$userRightsAssignment,
        [Parameter(Mandatory)][string]$account
    )

    $sid = Get-UserSid -userName $account

    Write-Result -Item "ローカルセキュリティポリシー - プロセスレベルトークンの置き換え権限($account)" -IsSuccess ($uraObject.HasProcessTokenPrivilege($sid))
    Write-Result -Item "ローカルセキュリティポリシー - プロセスのメモリクォータの増加($account)" -IsSuccess ($uraObject.HasIncreaseQuotaPrivilege($sid))
}

# ユーザー周りのチェック
function Test-UserRelationshipSettings {
    param(
        [Parameter(Mandatory)][string]$account
    )

    $user_account = Get-UserAccountInfo $account

    if ($user_account) {
        #監視アカウントがコンピューターの、Administratorsグループに属しているか確認する。
        Write-Result -Item "${account}ユーザーがAdministratorsグループに属している" -IsSuccess (Test-AdministratorGroupMembership $user_account)
        # ユーザーが有効化されているか確認する。
        if ($user_account.Disabled) {
            Write-Result -Item "${account}ユーザーが有効化されている" -IsSuccess $false
        }
    } else {
        Write-Result -Item "${account}ユーザーは存在しません。" -IsSuccess $false
    }
}

###############################################
# ここからメイン

$is_admin = TestAdministrator
if ( -not($true -eq $is_admin ) ) {
    Write-Error "管理者として実行してください。"
    Exit
}

# ユーザー周りの確認
Test-UserRelationshipSettings -account $account

#ユーザーアカウント制御(UAC)が無効化されているか確認する。
Write-Result -Item "UAC無効化確認" -IsSuccess (-Not(Get-UACEnabled))

#ローカルセキュリティポリシーのチェック
$uraObject = New-Object UserRightsAssignment
Test-SecurityPolicy -userRightsAssignment $uraObject -account "NT AUTHORITY\LocalService"
Test-SecurityPolicy -userRightsAssignment $uraObject -account "NT AUTHORITY\NetworkService"

#代理監視先がホスト名の場合名前解決できるか確認する。
if (-Not (Test-IPaddress -IPAddress $remote)) {
    Write-Result -Item "代理監視先の名前解決" -IsSuccess (Test-DnsResolution -Hostname $remote)
}

#代理監視先に対して以下のTCPポートに接続できるか確認する。
Write-Result -Item "代理監視先ポート接続確認 TCP445" -IsSuccess (Test-TcpPortOpen -Target $remote -Port 445)
Write-Result -Item "代理監視先ポート接続確認 TCP135" -IsSuccess (Test-TcpPortOpen -Target $remote -Port 135)

Exit